@@ -1,11 +1,10 @@
 from dataclasses import dataclass
 import numpy
 import numpy.typing
-from typing import Sequence, List, Tuple
+from typing import Sequence, List
 from pdme.measurement.dot_measure import DotMeasurement, DotRangeMeasurement
-
-
-DotInput = Tuple[numpy.typing.ArrayLike, float]
+from pdme.measurement.dot_pair_measure import DotPairMeasurement, DotPairRangeMeasurement
+from pdme.measurement.input_types import DotInput, DotPairInput
 
 
 @dataclass
@@ -53,15 +52,8 @@ class OscillatingDipole():
 	def _b(self, f: float) -> float:
 		return (1 / numpy.pi) * (self.w / (f**2 + self.w**2))
 
-
-def dot_inputs_to_array(dot_inputs: Sequence[DotInput]) -> numpy.ndarray:
-	return numpy.array([numpy.append(numpy.array(input[0]), input[1]) for input in dot_inputs])
-
-
-def dot_range_measurements_low_high_arrays(dot_range_measurements: Sequence[DotRangeMeasurement]) -> Tuple[numpy.ndarray, numpy.ndarray]:
-	lows = [measurement.v_low for measurement in dot_range_measurements]
-	highs = [measurement.v_high for measurement in dot_range_measurements]
-	return (numpy.array(lows), numpy.array(highs))
+	def s_for_dot_pair(self, r1: numpy.ndarray, r2: numpy.ndarray, f: float) -> float:
+		return self._alpha(r1) * self._alpha(r2) * self._b(f)
 
 
 class OscillatingDipoleArrangement():
@@ -80,12 +72,24 @@ class OscillatingDipoleArrangement():
 		f = dot_input[1]
 		return DotMeasurement(sum([dipole.s_at_position(r, f) for dipole in self.dipoles]), r, f)
 
+	def get_dot_pair_measurement(self, dot_pair_input: DotPairInput) -> DotPairMeasurement:
+		r1 = numpy.array(dot_pair_input[0])
+		r2 = numpy.array(dot_pair_input[1])
+		f = dot_pair_input[2]
+		return DotPairMeasurement(sum([dipole.s_for_dot_pair(r1, r2, f) for dipole in self.dipoles]), r1, r2, f)
+
 	def get_dot_measurements(self, dot_inputs: Sequence[DotInput]) -> List[DotMeasurement]:
 		'''
 		For a series of points, each with three coordinates and a frequency, return a list of the corresponding DotMeasurements.
 		'''
 		return [self.get_dot_measurement(dot_input) for dot_input in dot_inputs]
 
+	def get_dot_pair_measurements(self, dot_pair_inputs: Sequence[DotPairInput]) -> List[DotPairMeasurement]:
+		'''
+		For a series of pairs of points, each with three coordinates and a frequency, return a list of the corresponding DotPairMeasurements.
+		'''
+		return [self.get_dot_pair_measurement(dot_pair_input) for dot_pair_input in dot_pair_inputs]
+
 	def get_percent_range_dot_measurement(self, dot_input: DotInput, low_percent: float, high_percent: float) -> DotRangeMeasurement:
 		r = numpy.array(dot_input[0])
 		f = dot_input[1]
@@ -93,6 +97,18 @@ class OscillatingDipoleArrangement():
 
 	def get_percent_range_dot_measurements(self, dot_inputs: Sequence[DotInput], low_percent: float, high_percent: float) -> List[DotRangeMeasurement]:
 		'''
-		For a series of points, each with three coordinates and a frequency, and also a lower error range and upper error range, return a list of the corresponding DotRangeMeasurements.
+		For a series of pairs of points, each with three coordinates and a frequency, and also a lower error range and upper error range, return a list of the corresponding DotPairRangeMeasurements.
 		'''
 		return [self.get_percent_range_dot_measurement(dot_input, low_percent, high_percent) for dot_input in dot_inputs]
+
+	def get_percent_range_dot_pair_measurement(self, pair_input: DotPairInput, low_percent: float, high_percent: float) -> DotPairRangeMeasurement:
+		r1 = numpy.array(pair_input[0])
+		r2 = numpy.array(pair_input[1])
+		f = pair_input[2]
+		return DotPairRangeMeasurement(low_percent * sum([dipole.s_for_dot_pair(r1, r2, f) for dipole in self.dipoles]), high_percent * sum([dipole.s_for_dot_pair(r1, r2, f) for dipole in self.dipoles]), r1, r2, f)
+
+	def get_percent_range_dot_pair_measurements(self, pair_inputs: Sequence[DotPairInput], low_percent: float, high_percent: float) -> List[DotPairRangeMeasurement]:
+		'''
+		For a series of pairs of points, each with three coordinates and a frequency, and also a lower error range and upper error range, return a list of the corresponding DotPairRangeMeasurements.
+		'''
+		return [self.get_percent_range_dot_pair_measurement(pair_input, low_percent, high_percent) for pair_input in pair_inputs]
